@@ -28,76 +28,105 @@ For news, bugfixes, etc. visit the home page for this implementation at
     http://www.wayforward.net/spf/
 """
 
-# CVS Commits since last release (2.0.4):
+# CVS Commits since last release (2.0.5):
 # $Log: spf.py,v $
-# Revision 1.108.2.33  2008/04/23 21:00:42  customdesigned
-# Quote nulls in Received-SPF.
+# Revision 1.108.2.61  2011/10/27 16:29:38  customdesigned
+# Move python version test to def time.
 #
-# Revision 1.108.2.32  2008/04/23 20:03:53  customdesigned
-# Add timeout keyword to query constructor and DNSLookup.
+# Revision 1.108.2.60  2011/10/27 16:28:18  kitterma
+# Use bytes in to_ascii to work in python and python3.
 #
-# Revision 1.108.2.31  2008/03/27 01:15:33  customdesigned
-# Improve valid DNS name check.
+# Revision 1.108.2.59  2011/10/27 14:50:05  customdesigned
+# Ensure entire SPF policy is ascii.
 #
-# Revision 1.108.2.30  2008/03/27 00:58:15  customdesigned
-# Check dns names before DNSLookup
+# Revision 1.108.2.58  2011/10/27 14:29:49  customdesigned
+# Catch non-ascii domains.
 #
-# Revision 1.108.2.29  2008/03/26 15:08:20  kitterma
-# Fix commit log typo.
+# Revision 1.108.2.57  2011/10/27 10:32:06  kitterma
+# Drop version from spf.py shebang.
 #
-# Revision 1.108.2.28  2008/03/26 14:45:37  kitterma
-# Update built in tests for Python2.5 (addr2bin will now fail slightly with older
-# Python versions). SF #1655736
+# Revision 1.108.2.56  2011/10/27 04:58:03  kitterma
+# Update CHANGELOG, adjust minimum version requirement in setup.py, and update dates for a release.
 #
-# Revision 1.108.2.27  2008/03/26 14:34:35  kitterma
-# Change shebangs to #!/usr/bin/python throughout.
+# Revision 1.108.2.55  2011/10/27 03:49:11  kitterma
+# Fix doctests to raise ... as ... and print(x) as 2to3 doesn't fix these.
+# Doctests all pass in 2.6, 2.7, and 3.2
 #
-# Revision 1.108.2.26  2008/03/26 14:31:04  kitterma
-# Patch from Debian to avoid crash if command line SPF record request returns
-# TempError or PermError.
+# Revision 1.108.2.53  2011/10/18 02:56:32  kitterma
+# Resolve local conflicts in spf.py changelog.
 #
-# Revision 1.108.2.25  2008/03/26 14:26:19  kitterma
-# Update for new version (working on 2.0.5) and year.
+# Revision 1.108.2.52  2011/10/04 23:08:18  customdesigned
+# verbose option
 #
-# Revision 1.108.2.24  2008/03/24 21:33:22  customdesigned
-# Patch from Scott Kitterman to retry truncated results with TCP unless
-# in harsh mode.
+# Revision 1.108.2.51  2011/03/06 03:54:01  kitterma
+# Update copyright years.
 #
-# Revision 1.108.2.23  2007/11/28 19:48:37  customdesigned
-# Reflect decision on empty-exp errata.
+# Revision 1.108.2.50  2011/03/06 03:14:54  kitterma
+# Wrangle types around so addr2bin tests pass with python2.4/2.6/3.2(with 2to3).
 #
-# Revision 1.108.2.22  2007/06/23 20:17:09  customdesigned
-# Don't try to include null (None) keyword values.
+# Revision 1.108.2.49  2011/03/05 23:10:55  kitterma
+# Fix one missed instance of reverting to the older doctest with error type.
 #
-# Revision 1.108.2.21  2007/03/29 19:38:03  customdesigned
-# Remove trailing ';' again, fix Received-SPF tests.
+# Revision 1.108.2.48  2011/03/05 18:00:46  kitterma
+# Fix typo.
 #
-# Revision 1.108.2.20  2007/03/27 20:54:22  customdesigned
-# Correct Received-SPF header format.
+# Revision 1.108.2.47  2011/03/05 18:00:15  kitterma
+# Try to import both email.message and email.Message for backward compatibility.
 #
-# Revision 1.108.2.19  2007/03/17 19:07:01  customdesigned
-# For default modifier, return ambiguous in harsh mode, ignore in strict mode,
-# follow in lax mode.
+# Revision 1.108.2.46  2011/03/05 17:37:57  kitterma
+# Revert to older doctest construct for python2.4/2.5 compatibility and set minimum version to 2.4.
 #
-# Revision 1.108.2.18  2007/03/17 18:25:38  customdesigned
-# Default modifier is obsolete.  Retab (expandtab) spf.py
+# Revision 1.108.2.45  2011/03/03 04:14:31  kitterma
+#  * Refactor spf.py to support python3 via 2to3 - Minimum Python version is now python2.6.
+#  * Update README and CHANGELOG
 #
-# Revision 1.108.2.17  2007/03/13 20:13:16  customdesigned
-# Missing parentheses.
+# Revision 1.108.2.44  2011/02/11 18:25:31  kitterma
+# Move older spf.py commit messages to pyspf_changelog.txt and update version numbers.
+#
+# Revision 1.108.2.43  2011/02/11 18:17:47  kitterma
+# Ensure an error is raise for all DNS rcodes other than 0 and 3 per RFC 4408.
+#
+# Revision 1.108.2.42  2011/02/11 18:14:22  kitterma
+# Make TCP fallback an AmbiguityWarning in strict mode rather than an
+# error in harsh mode so we can retry and validate the TCP based record.
+#
+# Revision 1.108.2.41  2010/08/19 01:18:08  customdesigned
+# Return extra keyword dict from parse_header, parse identity.
+#
+# Revision 1.108.2.40  2010/04/29 20:23:44  customdesigned
+# Return result from parse_header
+#
+# Revision 1.108.2.39  2010/04/29 18:53:38  customdesigned
+# Parse Received-SPF header
+#
+# Revision 1.108.2.38  2010/04/29 16:36:47  customdesigned
+# report CIDR error only for valid mechanism
+#
+# Revision 1.108.2.37  2008/11/11 18:43:42  customdesigned
+# Make doc tests run on 2.5.  Heuristic for missing IP4.
+#
+# Revision 1.108.2.36  2008/09/10 00:46:45  customdesigned
+# Test case for handling invalid SPF on command line.
+#
+# Revision 1.108.2.35  2008/09/10 00:35:03  customdesigned
+# Handle invalid SPF record on command line.
+#
+# Revision 1.108.2.34  2008/08/25 17:58:07  customdesigned
+# Add timeout to check2.
 #
 # See pyspf_changelog.txt for earlier CVS commits.
 
 __author__ = "Terence Way"
 __email__ = "terry@wayforward.net"
-__version__ = "2.0.5: March 26, 2008"
+__version__ = "2.0.6: Oct 27, 2011"
 MODULE = 'spf'
 
 USAGE = """To check an incoming mail request:
-    % python spf.py {ip} {sender} {helo}
+    % python spf.py [-v] {ip} {sender} {helo}
     % python spf.py 69.55.226.139 tway@optsw.com mx1.wayforward.net
 
 To test an SPF record:
-    % python spf.py "v=spf1..." {ip} {sender} {helo}
+    % python spf.py [-v] "v=spf1..." {ip} {sender} {helo}
     % python spf.py "v=spf1 +mx +ip4:10.0.0.1 -all" 10.0.0.1 tway@foo.com a    
 
 To fetch an SPF record:
@@ -108,11 +137,17 @@ To test this script (and to output this usage message):
     % python spf.py
 """
 
+import sys
 import re
 import socket  # for inet_ntoa() and inet_aton()
 import struct  # for pack() and unpack()
 import time    # for time()
 import urllib  # for quote()
+import sys     # for version_info()
+try:
+    from email.message import Message
+except ImportError:
+    from email.Message import Message
 
 import DNS    # http://pydns.sourceforge.net
 if not hasattr(DNS.Type, 'SPF'):
@@ -133,12 +168,14 @@ def DNSLookup(name, qtype, strict=True, timeout=30):
         #
         if resp.header['tc'] == True:
           if strict > 1:
-              raise TempError, 'DNS: Truncated UDP Reply, not retrying TCP'
+              raise AmbiguityWarning, 'DNS: Truncated UDP Reply, SPF records should fit in a UDP packet, retrying TCP'
           try:
               req = DNS.DnsRequest(name, qtype=qtype, protocol='tcp')
               resp = req.req()
           except DNS.DNSError, x:
               raise TempError, 'DNS: TCP Fallback error: ' + str(x)
+          if resp.header['rcode'] != 0 and resp.header['rcode'] != 3:
+              raise IOError, 'Error: ' + resp.header['status'] + '  RCODE: ' + str(resp.header['rcode'])
         return [((a['name'], a['typename']), a['data']) for a in resp.answers]
     except IOError, x:
         raise TempError, 'DNS ' + str(x)
@@ -209,14 +246,6 @@ EXPLANATIONS = {'pass': 'sender SPF authorized',
                 'ambiguous': 'No error, but results may vary'
         }
 
-# support pre 2.2.1....
-try:
-    bool, True, False = bool, True, False
-except NameError:
-    False, True = 0, 1
-    def bool(x): return not not x
-# ...pre 2.2.1
-
 DELEGATE = None
 
 # standard default SPF record for best_guess
@@ -276,7 +305,7 @@ class PermError(Exception):
             return '%s: %s'%(self.msg, self.mech)
         return self.msg
 
-def check2(i, s, h, local=None, receiver=None):
+def check2(i, s, h, local=None, receiver=None, timeout=30, verbose=False):
     """Test an incoming MAIL FROM:<s>, from a client with ip address i.
     h is the HELO/EHLO domain name.  This is the RFC4408 compliant pySPF2.0
     interface.  The interface returns an SPF result and explanation only.
@@ -291,10 +320,11 @@ def check2(i, s, h, local=None, receiver=None):
     #>>> check2(i='61.51.192.42', s='liukebing@bcc.com', h='bmsi.com')
 
     """
-    res,_,exp = query(i=i, s=s, h=h, local=local, receiver=receiver).check()
+    res,_,exp = query(i=i, s=s, h=h, local=local,
+        receiver=receiver,timeout=timeout,verbose=verbose).check()
     return res,exp
 
-def check(i, s, h, local=None, receiver=None):
+def check(i, s, h, local=None, receiver=None, verbose=False):
     """Test an incoming MAIL FROM:<s>, from a client with ip address i.
     h is the HELO/EHLO domain name.  This is the pre-RFC SPF Classic interface.
     Applications written for pySPF 1.6/1.7 can use this interface to allow
@@ -308,7 +338,8 @@ def check(i, s, h, local=None, receiver=None):
     #>>> check(i='61.51.192.42', s='liukebing@bcc.com', h='bmsi.com')
 
     """
-    res,code,exp = query(i=i, s=s, h=h, local=local, receiver=receiver).check()
+    res,code,exp = query(i=i, s=s, h=h, local=local, receiver=receiver,
+        verbose=verbose).check()
     if res == 'permerror':
         res = 'unknown'
     elif res == 'tempfail':
@@ -337,7 +368,7 @@ class query(object):
     Also keeps cache: DNS cache.  
     """
     def __init__(self, i, s, h, local=None, receiver=None, strict=True,
-                timeout=30):
+                timeout=30,verbose=False):
         self.s, self.h = s, h
         if not s and h:
             self.s = 'postmaster@' + h
@@ -366,6 +397,10 @@ class query(object):
         if i:
             self.set_ip(i)
         self.default_modifier = True
+        self.verbose = verbose
+
+    def log(self,mech,d,spf):
+        print '%s: %s "%s"'%(mech,d,spf)
 
     def set_ip(self, i):
         "Set connect ip, and ip6 or ip4 mode."
@@ -508,6 +543,7 @@ class query(object):
             self.lookups = 0
             if not spf:
                 spf = self.dns_spf(self.d)
+                if self.verbose: self.log("top",self.d,spf)
             if self.libspf_local and spf: 
                 spf = insert_libspf_local_policy(
                     spf, self.libspf_local)
@@ -597,27 +633,27 @@ class query(object):
     ('?mx:%{d}/27', 'mx', 'email.example.com', 27, 'neutral')
 
     >>> try: q.validate_mechanism('ip4:1.2.3.4/247')
-    ... except PermError,x: print x
+    ... except PermError as x: print(x)
     Invalid IP4 CIDR length: ip4:1.2.3.4/247
     
     >>> try: q.validate_mechanism('ip4:1.2.3.4/33')
-    ... except PermError,x: print x
+    ... except PermError as x: print(x)
     Invalid IP4 CIDR length: ip4:1.2.3.4/33
 
     >>> try: q.validate_mechanism('a:example.com:8080')
-    ... except PermError,x: print x
+    ... except PermError as x: print(x)
     Invalid domain found (use FQDN): example.com:8080
     
     >>> try: q.validate_mechanism('ip4:1.2.3.444/24')
-    ... except PermError,x: print x
+    ... except PermError as x: print(x)
     Invalid IP4 address: ip4:1.2.3.444/24
     
     >>> try: q.validate_mechanism('ip4:1.2.03.4/24')
-    ... except PermError,x: print x
+    ... except PermError as x: print(x)
     Invalid IP4 address: ip4:1.2.03.4/24
     
     >>> try: q.validate_mechanism('-all:3030')
-    ... except PermError,x: print x
+    ... except PermError as x: print(x)
     Invalid all mechanism format - only qualifier allowed with all: -all:3030
 
     >>> q.validate_mechanism('-mx:%%%_/.Clara.de/27')
@@ -630,7 +666,7 @@ class query(object):
     ('a:mail.example.com.', 'a', 'mail.example.com', 32, 'pass')
 
     >>> try: q.validate_mechanism('a:mail.example.com,')
-    ... except PermError,x: print x
+    ... except PermError as x: print(x)
     Do not separate mechnisms with commas: a:mail.example.com,
     """
         if mech.endswith( "," ):
@@ -669,7 +705,10 @@ class query(object):
                 raise PermError('Invalid IP6 CIDR length', mech)
             if self.v == 'ip6':
                 cidrlength = cidr6length
-        elif m == 'ip4':
+        elif m == 'ip4' or RE_IP4.match(m):
+            if m != 'ip4':
+              self.note_error( 'Missing IP4' , mech)
+              m,arg = 'ip4',m
             if cidr6length is not None:
                 raise PermError('Dual CIDR not allowed', mech)
             if cidrlength is None:
@@ -689,6 +728,7 @@ class query(object):
                 raise PermError('Invalid IP6 address', mech)
         else:
             if cidrlength is not None or cidr6length is not None:
+              if m in ALL_MECHANISMS:
                 raise PermError('CIDR not allowed', mech)
             cidrlength = self.cidrmax
 
@@ -738,8 +778,9 @@ class query(object):
         # for common mistakes like IN TXT "v=spf1" "mx" "-all"
         # in relaxed mode.
         if spf[0].lower() != 'v=spf1':
-            assert strict > 1
-            raise AmbiguityWarning('Invalid SPF record in', self.d)
+            if self.strict > 1:
+                raise AmbiguityWarning('Invalid SPF record in', self.d)
+            return ('none', 250, EXPLANATIONS['none'])
         spf = spf[1:]
 
         # copy of explanations to be modified by exp=
@@ -808,8 +849,10 @@ class query(object):
 
             if m == 'include':
                 self.check_lookups()
-                res, code, txt = self.check1(self.dns_spf(arg),
-                      arg, recursion + 1)
+                d = self.dns_spf(arg)
+                if self.verbose: self.log("include",arg,d)
+                #print "%s: %s"%(arg,d)
+                res, code, txt = self.check1(d,arg, recursion + 1)
                 if res == 'pass':
                     break
                 if res == 'none':
@@ -869,6 +912,7 @@ class query(object):
                 if not redirect_record:
                     raise PermError('redirect domain has no SPF record',
                         redirect)
+                if self.verbose: self.log("redirect",redirect,redirect_record)
                 # forget modifiers on redirect
                 if not recursion:
                   self.exps = dict(self.defexps)
@@ -977,7 +1021,7 @@ class query(object):
         '3.2.0.192.in-addr.strong.lp._spf.example.com'
 
         >>> try: q.expand('%(ir).%{v}.%{l1r-}.lp._spf.%{d2}')
-        ... except PermError,x: print x
+        ... except PermError as x: print(x)
         invalid-macro-char : %(ir)
 
         >>> q.expand('%{p2}.trusted-domains.example.net')
@@ -1059,7 +1103,7 @@ class query(object):
         if len(a) > 1:
             raise PermError('Two or more type TXT spf records found.')
         if len(a) == 1 and self.strict < 2:
-            return a[0]               
+            return to_ascii(a[0])
         # check official SPF type first when it becomes more popular
         if self.strict > 1:
             #Only check for Type SPF in harsh mode until it is more popular.
@@ -1076,15 +1120,15 @@ class query(object):
                 #Changed from permerror to warning based on RFC 4408 Auth 48 change
                     raise AmbiguityWarning(
 'v=spf1 records of both type TXT and SPF (type 99) present, but not identical')
-                return b[0]
+                return to_ascii(b[0])
         if len(a) == 1:
-            return a[0]    # return TXT if SPF wasn't found
+            return to_ascii(a[0])    # return TXT if SPF wasn't found
         if DELEGATE:    # use local record if neither found
             a = [t
               for t in self.dns_txt(domain+'._spf.'+DELEGATE)
             if RE_SPF.match(t)
             ]
-            if len(a) == 1: return a[0]
+            if len(a) == 1: return to_ascii(a[0])
         return None
 
     def dns_txt(self, domainname):
@@ -1230,7 +1274,55 @@ class query(object):
         except socket.error: pass
         return False
 
+    def parse_header(self, val):
+        """Set SPF values from Received-SPF header.
+        
+        Useful when SPF has already been run on a trusted gateway machine.
+
+        Examples:
+        >>> q = query('0.0.0.0','','')
+        >>> p = q.parse_header('''Pass (test) client-ip=70.98.79.77;
+        ... envelope-from="evelyn@subjectsthum.com"; helo=mail.subjectsthum.com;
+        ... receiver=mail.bmsi.com; mechanism=a; identity=mailfrom''')
+        >>> q.get_header(q.result)
+        'Pass (test) client-ip=70.98.79.77; envelope-from="evelyn@subjectsthum.com"; helo=mail.subjectsthum.com; receiver=mail.bmsi.com; mechanism=a; identity=mailfrom'
+	>>> p = q.parse_header('''None (mail.bmsi.com: test)
+	... client-ip=163.247.46.150; envelope-from="admin@squiebras.cl";
+	... helo=mail.squiebras.cl; receiver=mail.bmsi.com; mechanism=mx/24;
+	... x-bestguess=pass; x-helo-spf=neutral; identity=mailfrom''')
+	>>> q.get_header(q.result,**p)
+	'None (mail.bmsi.com: test) client-ip=163.247.46.150; envelope-from="admin@squiebras.cl"; helo=mail.squiebras.cl; receiver=mail.bmsi.com; mechanism=mx/24; x-bestguess=pass; x-helo-spf=neutral; identity=mailfrom'
+	>>> p['bestguess']
+	'pass'
+        """
+        a = val.split(None,1)
+        self.result = a[0].lower()
+        self.mechanism = None
+        if len(a) < 2: return 'none'
+        val = a[1]
+        if val.startswith('('):
+          pos = val.find(')')
+          if pos < 0: return self.result
+          self.comment = val[1:pos]
+          val = val[pos+1:]
+        msg = Message()
+        msg.add_header('Received-SPF','; '+val)
+        p = {}
+        for k,v in msg.get_params(header='Received-SPF'):
+          if k == 'client-ip':
+            self.set_ip(v)
+          elif k == 'envelope-from': self.s = v
+          elif k == 'helo': self.h = v
+          elif k == 'receiver': self.r = v
+          elif k == 'problem': self.mech = v
+          elif k == 'mechanism': self.mechanism = v
+          elif k == 'identity': self.ident = v
+          elif k.startswith('x-'): p[k[2:]] = v
+        self.l, self.o = split_email(self.s, self.h)
+        return p
+
     def get_header(self, res, receiver=None, **kv):
+        """Generate Received-SPF header based on last lookup."""
         if not receiver:
             receiver = self.r
         client_ip = self.c
@@ -1249,7 +1341,11 @@ class query(object):
         else:
             problem = None
         mechanism = quote_value(self.mechanism)
-        res = ['%s (%s: %s)' % (tag,receiver,self.get_header_comment(res))]
+        if hasattr(self,'comment'):
+          comment = self.comment
+        else:
+          comment = '%s: %s' % (receiver,self.get_header_comment(res))
+        res = ['%s (%s)' % (tag,comment)]
         for k in ('client_ip','envelope_from','helo','receiver',
           'problem','mechanism'):
             v = locals()[k]
@@ -1261,8 +1357,7 @@ class query(object):
         return ' '.join(res)
 
     def get_header_comment(self, res):
-        """Return comment for Received-SPF header.
-        """
+        """Return comment for Received-SPF header.  """
         sender = self.o
         if res == 'pass':
             return \
@@ -1440,28 +1535,36 @@ def addr2bin(str):
     """Convert a string IPv4 address into an unsigned integer.
 
     Examples::
-    >>> addr2bin('127.0.0.1')
+    >>> import sys
+    >>> if sys.version_info[0] == 2:
+    ...     print(long(addr2bin('127.0.0.1')))
+    ... else:
+    ...     print(addr2bin('127.0.0.1'))
     2130706433
 
     >>> addr2bin('127.0.0.1') == socket.INADDR_LOOPBACK
     1
 
-    >>> addr2bin('255.255.255.254')
-    4294967294L
+    >>> print(addr2bin('255.255.255.254'))
+    4294967294
 
-    >>> addr2bin('192.168.0.1')
-    3232235521L
+    >>> print(addr2bin('192.168.0.1'))
+    3232235521
 
     Unlike DNS.addr2bin, the n, n.n, and n.n.n forms for IP addresses
     are handled as well::
-    >>> addr2bin('10.65536')
-    167837696
-    >>> 10 * (2 ** 24) + 65536
+    >>> import sys
+    >>> if sys.version_info[0] == 2:
+    ...     print(long(addr2bin('10.65536')))
+    ... else:
+    ...     print(addr2bin('10.65536'))
     167837696
 
-    >>> addr2bin('10.93.512')
-    173867520
-    >>> 10 * (2 ** 24) + 93 * (2 ** 16) + 512
+    >>> import sys
+    >>> if sys.version_info[0] == 2:
+    ...     print(long(addr2bin('10.93.512')))
+    ... else:
+    ...     print(addr2bin('10.93.512'))
     173867520
     """
     return struct.unpack("!L", socket.inet_aton(str))[0]
@@ -1655,6 +1758,21 @@ def insert_libspf_local_policy(spftxt, local=None):
     #MAX_LOOKUP = 100 
     return 'v=spf1 '+local
 
+if sys.version_info[0] == 2:
+  def to_ascii(s):
+      "Raise PermError is arg is not 7-bit ascii."
+      try:
+        return s.encode('ascii')
+      except UnicodeEncodeError:
+        raise PermError('Non-ascii domain found',repr(s))
+else:
+  def to_ascii(s):
+      "Raise PermError is arg is not 7-bit ascii."
+      try:
+        return bytes(s,'ascii').decode('ascii')
+      except UnicodeEncodeError:
+        raise PermError('Non-ascii domain found',repr(s))
+
 def _test():
     import doctest, spf
     return doctest.testmod(spf)
@@ -1662,27 +1780,39 @@ def _test():
 DNS.DiscoverNameServers() # Fails on Mac OS X? Add domain to /etc/resolv.conf
 
 if __name__ == '__main__':
-    import sys
-    if len(sys.argv) == 1:
+    import getopt
+    try:
+       opts,argv = getopt.getopt(sys.argv[1:],"hv",["help","verbose"])
+    except getopt.GetoptError, err:
+       print str(err)
+       print USAGE
+       sys.exit(2)
+    verbose = False
+    for o,a in opts:
+        if o in ('-v','--verbose'):
+           verbose = True
+        elif o in ('-h','--help'):
+           print USAGE
+    if len(argv) == 0:
         print USAGE
         _test()
-    elif len(sys.argv) == 2:
+    elif len(argv) == 1:
         try:
             q = query(i='127.0.0.1', s='localhost', h='unknown',
                 receiver=socket.gethostname())
-            print q.dns_spf(sys.argv[1])
+            print q.dns_spf(argv[0])
         except TempError, x:
             print "Temporary DNS error: ", x
         except PermError, x:
             print "PermError: ", x
-    elif len(sys.argv) == 4:
-        print check(i=sys.argv[1], s=sys.argv[2], h=sys.argv[3],
-            receiver=socket.gethostname())
-    elif len(sys.argv) == 5:
-        i, s, h = sys.argv[2:]
+    elif len(argv) == 3:
+        print check(i=argv[0], s=argv[1], h=argv[2],
+            receiver=socket.gethostname(), verbose=verbose)
+    elif len(argv) == 4:
+        i, s, h = argv[1:]
         q = query(i=i, s=s, h=h, receiver=socket.gethostname(),
-            strict=False)
-        print q.check(sys.argv[1]),q.mechanism
+            strict=False, verbose=verbose)
+        print q.check(argv[0]),q.mechanism
         if q.perm_error and q.perm_error.ext:
             print q.perm_error.ext
     else:
